// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
	/// </summary>
	[Description("The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.")]
	public class RSI_DataSeriesInput : Indicator
	{
		#region Variables
		private DataSeries	pTheInput;
		private DataSeries	avgUp;
		private DataSeries	avgDown;
		private DataSeries	down;
		private int			period	= 14;
		private int			smooth	= 3;
		private DataSeries	up;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Green, "RSI_DataSeriesInput"));
			Add(new Plot(Color.Orange, "Avg"));

			Add(new Line(System.Drawing.Color.DarkViolet, 30, "Lower"));
			Add(new Line(System.Drawing.Color.YellowGreen, 70, "Upper"));

			avgUp				= new DataSeries(this);
			avgDown				= new DataSeries(this);
			down				= new DataSeries(this);
			up					= new DataSeries(this);

			PriceTypeSupported	= true;
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar == 0)
			{
				down.Set(0);
				up.Set(0);
				return;
			}

			down.Set(Math.Max(pTheInput[1] - pTheInput[0], 0));
			up.Set(Math.Max(pTheInput[0] - pTheInput[1], 0));

			if ((CurrentBar + 1) < Period) 
			{
				if ((CurrentBar + 1) == (Period - 1))
					Avg.Set(50);
				return;
			}

			if ((CurrentBar + 1) == Period) 
			{
				// First averages 
				avgDown.Set(SMA(down, Period)[0]);
				avgUp.Set(SMA(up, Period)[0]);
			}  
			else 
			{
				// Rest of averages are smoothed
				avgDown.Set((avgDown[1] * (Period - 1) + down[0]) / Period);
				avgUp.Set((avgUp[1] * (Period - 1) + up[0]) / Period);
			}

			double rs	  = avgUp[0] / (avgDown[0] == 0 ? 1 : avgDown[0]);
			double rsi	  = 100 - (100 / (1 + rs));
			double rsiAvg = (2.0 / (1 + Smooth)) * rsi + (1 - (2.0 / (1 + Smooth))) * Avg[1];

			Avg.Set(rsiAvg);
			Value.Set(rsi);
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Avg
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("TheDataSeries input")]
		[Category("Parameters")]
		public DataSeries TheInput
		{
			get { return pTheInput; }
			set { pTheInput = value; }
		}
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(2, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RSI_DataSeriesInput[] cacheRSI_DataSeriesInput = null;

        private static RSI_DataSeriesInput checkRSI_DataSeriesInput = new RSI_DataSeriesInput();

        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSI_DataSeriesInput RSI_DataSeriesInput(int period, int smooth, DataSeries theInput)
        {
            return RSI_DataSeriesInput(Input, period, smooth, theInput);
        }

        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSI_DataSeriesInput RSI_DataSeriesInput(Data.IDataSeries input, int period, int smooth, DataSeries theInput)
        {
            checkRSI_DataSeriesInput.Period = period;
            period = checkRSI_DataSeriesInput.Period;
            checkRSI_DataSeriesInput.Smooth = smooth;
            smooth = checkRSI_DataSeriesInput.Smooth;
            checkRSI_DataSeriesInput.TheInput = theInput;
            theInput = checkRSI_DataSeriesInput.TheInput;

            if (cacheRSI_DataSeriesInput != null)
                for (int idx = 0; idx < cacheRSI_DataSeriesInput.Length; idx++)
                    if (cacheRSI_DataSeriesInput[idx].Period == period && cacheRSI_DataSeriesInput[idx].Smooth == smooth && cacheRSI_DataSeriesInput[idx].TheInput == theInput && cacheRSI_DataSeriesInput[idx].EqualsInput(input))
                        return cacheRSI_DataSeriesInput[idx];

            RSI_DataSeriesInput indicator = new RSI_DataSeriesInput();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;
            indicator.Smooth = smooth;
            indicator.TheInput = theInput;
            indicator.SetUp();

            RSI_DataSeriesInput[] tmp = new RSI_DataSeriesInput[cacheRSI_DataSeriesInput == null ? 1 : cacheRSI_DataSeriesInput.Length + 1];
            if (cacheRSI_DataSeriesInput != null)
                cacheRSI_DataSeriesInput.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRSI_DataSeriesInput = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSI_DataSeriesInput RSI_DataSeriesInput(int period, int smooth, DataSeries theInput)
        {
            return _indicator.RSI_DataSeriesInput(Input, period, smooth, theInput);
        }

        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSI_DataSeriesInput RSI_DataSeriesInput(Data.IDataSeries input, int period, int smooth, DataSeries theInput)
        {
            return _indicator.RSI_DataSeriesInput(input, period, smooth, theInput);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSI_DataSeriesInput RSI_DataSeriesInput(int period, int smooth, DataSeries theInput)
        {
            return _indicator.RSI_DataSeriesInput(Input, period, smooth, theInput);
        }

        /// <summary>
        /// The RSI_DataSeriesInput (Relative Strength Index) is a price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSI_DataSeriesInput RSI_DataSeriesInput(Data.IDataSeries input, int period, int smooth, DataSeries theInput)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RSI_DataSeriesInput(input, period, smooth, theInput);
        }

    }
}
#endregion
